#!/usr/bin/python2.7

import unittest
import stat
import os

from setup import *
from sandglass import cache

class CacheTest(TestCaseTempDir):
	"""
	Module sandglass.cache Unit Test.
	"""

	def test_cache_creates_directory(self):
		"""Cache directory should be created if it not exists."""

		location = os.path.join(self.temp_dir, "cache")
		cache.set_location(location)

		self.assertFalse(os.path.isdir(location))
		cache.set_location(location)
		foo = cache.Cache("foo")
		self.assertTrue(os.path.isdir(location))

	def test_cache_data_file_created(self):
		"""Datafile should be created in the cache directory upon save."""

		cache.set_location(self.temp_dir)
		filename_foo = os.path.join(self.temp_dir, "foo")
		filename_bar = os.path.join(self.temp_dir, "bar")

		self.assertFalse(os.path.exists(filename_foo))
		self.assertFalse(os.path.exists(filename_bar))

		foo = cache.Cache("foo")
		bar = cache.Cache("bar")

		self.assertFalse(os.path.exists(filename_foo))
		self.assertFalse(os.path.exists(filename_bar))

		foo.data = "nothing important"	# first possibility
		bar.save()			# second possibility

		self.assertTrue(os.path.exists(filename_foo))
		self.assertTrue(os.path.exists(filename_bar))

	def test_cache_data_none_if_new(self):
		"""Data should be None when the cache is created."""

		cache.set_location(self.temp_dir)

		foo = cache.Cache("foo")
		self.assertIsNone(foo.data)

	def test_cache_data_autosave(self):
		"""Data are saved to cache and can be loaded."""

		cache.set_location(self.temp_dir)

		sample = {
			"a" : "string",
			"b" : 12345,
			"c" : [ "a", 42 ],
			"d" : { "x" : -13, "y" : 3.2 },
			"e" : ( 8, 7, 5 )
		}

		foo = cache.Cache("foo")
		foo.data = sample

		del foo
		new_foo = cache.Cache("foo")
		stored  = new_foo.data

		self.assertEqual(sample, stored)

	def test_cache_data_manual_save(self):
		"""Bound objects can be saved manually using 'save' method."""

		cache.set_location(self.temp_dir)

		sample_array = [ "bum", "ta", "rata", "na", "vrata" ]

		foo = cache.Cache("foo")
		foo.data = sample_array

		sample_array.reverse()

		other_foo = cache.Cache("foo")
		self.assertNotEqual(sample_array, other_foo.data)
		del other_foo

		foo.save()

		other_foo = cache.Cache("foo")
		self.assertEqual(sample_array, other_foo.data)

	def test_cache_file_permissions(self):
		"""Cache file permissions can be set via public= option."""

		read_bits =  (stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
		write_bits = (stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH)

		cache.set_location(self.temp_dir)

		# public cache file (foo)

		foo_file = os.path.join(self.temp_dir, "foo")

		self.assertFalse(os.path.exists(foo_file))
		foo = cache.Cache("foo", public=True)
		foo.data = 111
		self.assertTrue(os.path.exists(foo_file))

		foo_perm = stat.S_IMODE(os.stat(foo_file).st_mode)
		self.assertTrue((foo_perm & read_bits) == read_bits)
		self.assertTrue((foo_perm & write_bits) == stat.S_IWUSR)

		del foo, foo_file, foo_perm

		# non-public cache file (bar)

		bar_file = os.path.join(self.temp_dir, "bar")

		self.assertFalse(os.path.exists(bar_file))
		bar = cache.Cache("bar", public=False)
		bar.data = 222
		self.assertTrue(os.path.exists(bar_file))

		bar_perm = stat.S_IMODE(os.stat(bar_file).st_mode)
		self.assertTrue((bar_perm & read_bits) == stat.S_IRUSR)
		self.assertTrue((bar_perm & write_bits) == stat.S_IWUSR)

	def test_cache_saved_at_works(self):
		"""Method 'saved_at' should correspond with file mtime."""

		cache.set_location(self.temp_dir)
		location = os.path.join(self.temp_dir, "foo")

		# not saved
		foo = cache.Cache("foo")
		self.assertEqual(foo.saved_at(), 0.0)

		# after save
		foo.save()
		self.assertEqual(foo.saved_at(), os.lstat(location).st_mtime)

		# manually set atime, mtime
		os.utime(location, (13.0, 42.0))
		self.assertEqual(foo.saved_at(), 42.0)

if __name__ == "__main__":
	unittest.main()
