#!/usr/bin/python2.7

import unittest
from mocker import Mocker
import os
import stat

from setup import *
from sandglass import utils

class UtilsTest(TestCaseTempDir):
	"""
	Module sandglass.utils Unit Test.
	"""

	def test_abs_path_absolute(self):
		"""Method 'abs_path' should return valid absolute and normalized paths for absolute paths at input."""

		self.assertEqual(utils.abs_path("/"), "/")
		self.assertEqual(utils.abs_path("/etc"), "/etc")
		self.assertEqual(utils.abs_path("/home/"), "/home")
		self.assertEqual(utils.abs_path("/var///"), "/var")
		self.assertEqual(utils.abs_path("/../../sys/"), "/sys")
		self.assertEqual(utils.abs_path("/etc/../etc/../etc"), "/etc")
		self.assertEqual(utils.abs_path("/home/../tmp/../var/file"), "/var/file")
		self.assertEqual(utils.abs_path("/lib/.././../lib64/././dir/././"), "/lib64/dir")
		self.assertEqual(utils.abs_path("/etc/./sysconfig///network"), "/etc/sysconfig/network")

	def test_abs_path_relatve(self):
		"""Method 'abs_path' should return valid absolute and normalized paths for relative paths at input."""

		# this might be tricky
		cwd = os.getcwd()
		self.assertEqual(utils.abs_path("../../../../../"), "/")
		self.assertEqual(utils.abs_path(cwd), cwd)
		self.assertEqual(utils.abs_path(os.path.join(cwd, ".", ".", ".")), cwd)
		self.assertEqual(utils.abs_path(os.path.join(cwd, "foo", "..", "bar")), os.path.join(cwd, "bar"))

	def test_is_dir(self):
		"""Method 'is_dir' should return True only if the path is real directory."""

		# filenames
		filename_nothing = os.path.join(self.temp_dir, "i_am_not")
		filename_file = os.path.join(self.temp_dir, "i_am_a_file")
		filename_directory = os.path.join(self.temp_dir, "i_am_a_directory")
		filename_symlink_dangling = os.path.join(self.temp_dir, "i_am_wrong_link")
		filename_symlink_file = os.path.join(self.temp_dir, "i_am_file_link")
		filename_symlink_directory = os.path.join(self.temp_dir, "i_am_directory_link")

		# create content
		open(filename_file, "w").write("some content")
		os.mkdir(filename_directory)
		os.symlink(filename_nothing, filename_symlink_dangling)
		os.symlink(filename_file, filename_symlink_file)
		os.symlink(filename_directory, filename_symlink_directory)

		# test result
		self.assertFalse(utils.is_dir(filename_nothing))
		self.assertFalse(utils.is_dir(filename_file))
		self.assertTrue(utils.is_dir(filename_directory))
		self.assertFalse(utils.is_dir(filename_symlink_dangling))
		self.assertFalse(utils.is_dir(filename_symlink_file))
		self.assertFalse(utils.is_dir(filename_symlink_directory))

	def test_files_from_dir(self):
		"""Method 'files_from_dir' should return all files in a subtree."""

		files = [ "apple", "banana", "cherry/date", "cherry/fig", "grape/kiwi/lemon/mango", "plum" ]
		files = map(lambda name: os.path.join(self.temp_dir, name), files)

		# create files
		for filename in files:
			dirname = os.path.dirname(filename)
			if not os.path.exists(dirname):
				os.makedirs(dirname)
			open(filename, "w").write("%s juice" % filename)

		result = utils.files_from_dir(self.temp_dir)
		self.assertTrue(sorted(result) == sorted(files))

	def test_hostname_fqdn(self):
		"""Method 'hostname' should return system host name if 'socket.getfqdn()' is not 'localhost'."""

		mocker = Mocker()
		getfqdn = mocker.replace("socket.getfqdn")
		getfqdn()
		mocker.result("happyfedora")
		with mocker:
			result = utils.hostname()

		self.assertEquals(result, "happyfedora")

	def test_hostname_gethostname(self):
		"""Method 'hostname' should return 'socket.gethostname()' if 'socket.getfqdn()' returns 'localhost'."""

		mocker = Mocker()
		getfqdn = mocker.replace("socket.getfqdn")
		getfqdn()
		mocker.result("localhost")
		gethostname = mocker.replace("socket.gethostname")
		gethostname()
		mocker.result("freefedora")
		with mocker:
			result = utils.hostname()

		self.assertEquals(result, "freefedora")

	def test_hostname_localhost(self):
		"""Method 'hostname' should return 'localhost' if both 'socket.getfqdn()' and 'socket.gethostname()' returns is."""

		mocker = Mocker()
		getfqdn = mocker.replace("socket.getfqdn")
		getfqdn()
		mocker.result("localhost")
		gethostname = mocker.replace("socket.gethostname")
		gethostname()
		mocker.result("localhost")
		with mocker:
			result = utils.hostname()

		self.assertEquals(result, "localhost")

	def test_file_mtime(self):
		"""Method 'file_mtime' should work for files and directories."""

		filename_file = os.path.join(self.temp_dir, "file")
		filename_dir = os.path.join(self.temp_dir, "dir")

		open(filename_file, "w").write("file content")
		os.mkdir(filename_dir)

		os.utime(filename_file, (42.0, 13.0))
		os.utime(filename_dir, (13.0, 42.0))

		self.assertTrue(utils.file_mtime(filename_file, 13.0))
		self.assertTrue(utils.file_mtime(filename_dir, 32.0))

	def test_file_mtime_exists(self):
		"""Method 'file_mtime' should return 0.0 if file not exists and 'must_exist' argument is False'."""

		filename = os.path.join(self.temp_dir, "foo")

		self.assertRaises(OSError, utils.file_mtime, filename)
		self.assertRaises(OSError, utils.file_mtime, filename, must_exist=True)
		self.assertEquals(utils.file_mtime(filename, must_exist=False), 0.0)

	def test_file_mtime_symlinks(self):
		"""Method 'file_mtime' should work for symlinks, which must not be followed."""

		filename_file = os.path.join(self.temp_dir, "wine")
		filename_link = os.path.join(self.temp_dir, "beer")

		open(filename_file, "w").write("good wine inside")
		os.symlink(filename_file, filename_link)

		# os.utime always dereferences symlinks... :-(
		os.system("touch -h -t 197001010101.00 %s" % filename_file)
		os.system("touch -h -t 197001010102.00 %s" % filename_link)

		self.assertEquals(utils.file_mtime(filename_file), 60.0)
		self.assertEquals(utils.file_mtime(filename_link), 120.0)

	def test_temp_file(self):
		"""Method 'temp_file' should create temporary file and delete it using atexit."""

		tempfile = os.path.join(self.temp_dir, "our_temp_file")
		tempfile_fd = 33

		# create temp file
		mocker = Mocker()
		tempfile_mkstemp = mocker.replace("tempfile.mkstemp")
		tempfile_mkstemp()
		mocker.result((tempfile_fd, tempfile))
		# register atexit
		atexit_register = mocker.replace("atexit.register")
		atexit_register(utils._temp_file_delete, tempfile)
		mocker.result(None)
		# close descriptor
		os_close = mocker.replace("os.close")
		os_close(tempfile_fd)
		mocker.result(None)

		with mocker:
			result = utils.temp_file()

		self.assertTrue(result, tempfile)

	def test_secure_open(self):
		"""Method 'secure_open' should open user-accessible only file."""

		read_bits =  (stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
		write_bits = (stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH)
		filename = os.path.join(self.temp_dir, "secure_file")

		fh = utils.secure_open(filename, "w")

		perms = stat.S_IMODE(os.stat(filename).st_mode)
		self.assertTrue((perms & read_bits) == stat.S_IRUSR)
		self.assertTrue((perms & write_bits) == stat.S_IWUSR)

		fh.close()

	def test_secure_makedirs(self):
		"""Method 'secure_makedirs' should create user-accessible only directories."""

		read_bits =  (stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
		write_bits = (stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH)

		path_a = os.path.join(self.temp_dir, "tomato")
		path_b = os.path.join(path_a, "carrot")
		path_c = os.path.join(path_b, "cabbage")

		utils.secure_makedirs(path_c)
		for path in [path_a, path_b, path_c]:
			perms = stat.S_IMODE(os.stat(path).st_mode)
			self.assertTrue((perms & read_bits) == stat.S_IRUSR)
			self.assertTrue((perms & write_bits) == stat.S_IWUSR)

	def test_time_remains_and_time_ago(self):
		"""Methods 'time_remains' and 'time_ago' should return correct value."""

		mocker = Mocker()
		time = mocker.replace("time.time")
		time()
		mocker.result(1000.0)
		mocker.count(1, None)
		with mocker:
			# time_remains
			self.assertEquals(utils.time_remains(2000), 1000)
			self.assertEquals(utils.time_remains(1000), 0)
			self.assertEquals(utils.time_remains(-1000), -2000)
			# time_ago
			self.assertEquals(utils.time_ago(2000), -1000)
			self.assertEquals(utils.time_ago(1000), 0)
			self.assertEquals(utils.time_ago(-1000), 2000)

	def test_time_relative(self):
		"""Method 'time_relative' should return appropriate textual representation."""

		mocker = Mocker()
		time = mocker.replace("time.time")
		time()
		mocker.result(1000.0)
		mocker.count(1, None)
		with mocker:
			# time_remains
			self.assertEquals(utils.time_relative(1000), "now")
			self.assertEquals(utils.time_relative(1001), "in 1 second")
			self.assertEquals(utils.time_relative(1005), "in 5 seconds")
			self.assertEquals(utils.time_relative(999), "1 second ago")
			self.assertEquals(utils.time_relative(995), "5 seconds ago")
			self.assertEquals(utils.time_relative(1065), "in 1 minute")
			self.assertEquals(utils.time_relative(1130), "in 2 minutes")
			self.assertEquals(utils.time_relative(940), "1 minute ago")
			self.assertEquals(utils.time_relative(840), "2 minutes ago")
			self.assertEquals(utils.time_relative(5000), "in 1 hour")
			self.assertEquals(utils.time_relative(10000), "in 2 hours")
			self.assertEquals(utils.time_relative(-5000), "1 hour ago")
			self.assertEquals(utils.time_relative(-10000), "3 hours ago")
			self.assertEquals(utils.time_relative(100000), "in 1 day")
			self.assertEquals(utils.time_relative(400000), "in 4 days")
			self.assertEquals(utils.time_relative(-100000), "1 day ago")
			self.assertEquals(utils.time_relative(-400000), "4 days ago")

	def test_timestamp_to_str(self):
		"""Method 'timestamp_to_str' should return correct results."""

		self.assertEquals(utils.timestamp_to_str(1000), "1970-01-01 01:16:40")
		self.assertEquals(utils.timestamp_to_str(1e9), "2001-09-09 03:46:40")
		self.assertEquals(utils.timestamp_to_str(2e9), "2033-05-18 05:33:20")

if __name__ == "__main__":
	unittest.main()
